/////////////////////////////////////////////////////////////////////////////
//  Projekt: STest - testovn slovek a tvorba test                     //
//  Verze: 2                                                               //
//  Autor: David Martinek                                                  //
//  Adresa: Majakovskho 2148/60, Karvin 7, 734 01                        //
//                                                                         //
//  Funkce modulu: Prce se seznamem slovek                              //
//  Datum: 8.9.1999                                                       //
/////////////////////////////////////////////////////////////////////////////
unit WordList;
// {$DEFINE debug}
interface

uses Classes;

const
  WordSeparator : char = #16;     // oddlova jednotlivch slov
  SingleSeparator : char = #19;   // oddlova esk a anglick sti pru
                                  // v souboru
  PairSeparator : char = #10;     // oddlova pr v souboru
  CommentPrep : char = #12;       // pedloka komente

type
  TActualMode = (amOrigin, amTrans, amMix); // aktuln md zkouen

  TPair = class      // pr slovek
  public
    Origin,          // seznam slovek v originle
    Trans : String;  // seznam odpovdajcch peloench slovek
    OriginComment,          // koment k originlu
    TransComment : String;  // koment k pekladu
{$IFDEF debug}
    destructor Destroy; override; // testovac destruktor
{$ENDIF}
    // vrt Origin nebo Trans
    function GetWordList(Mode : TActualMode) : String;
    // vrt koment k orginlu nebo k pekladu
    function GetComment(Mode :TActualMode) : String;
    // vrt poet slov v originle nebo v pekladu
    function GetNum(Mode : TActualMode) : integer;
    // vrt slovo z originlu nebo pekladu
    function GetWord(Mode : TActualMode; Index : integer) : String;
    // hled slovo S v originle nebo pekladu
    function Find(Mode : TActualMode; S : String) : boolean;
  end;

  // tda realizujc seznam pr slovek
  TWordList = class (TList)
  private
    FError : integer;
    ch : char;        // slou pro uchovn jednoho znaku ze souboru
    // vrac seznam slovek, tj. originl nebo peklad
    function GetWord(var F : TextFile) : String;
  public
    constructor Create;
    destructor Destroy; override;
    // zkopruje se do jinho seznamu; parametr u mus bt vytvoen
//    procedure CopyList(var L : TWordList);
    // statick funkce pro vracen textovho popisu chyby
    class function GetErrorText(E : integer) : String;
    // vrac aktuln kd chyby
    function GetError : integer;
    // pid pr do seznamu
    function Add(Item: TPair): Integer;
    // odstran pr ze seznamu
    function Remove(Item: TPair): Integer;
    // ulo seznam do souboru
    procedure SaveToFile(FileName : String);
    // nate seznam ze souboru
    procedure LoadFromFile(FileName : String);
    // vrt poet originl nebo peklad
    function GetCount(Mode : TActualMode) : integer;
  end; // TWordList

TIndexItem = record
  windex,                // index prvku v seznamu slovek
  number : integer;     // poad slovka
  mode : TActualMode;   // uruje jestli je slovko originl nebo peklad
end;

TIndexArray = array [0..0] of TIndexItem;
PIndexArray = ^TIndexArray;
TIndexList = class
protected
  IndexArray : PIndexArray;
  Size : integer;
public
  Count : integer;
  Maximum : integer;
  Index : integer;

  constructor Create(const L : TWordList; Test : TActualMode; Max : integer);

  destructor Destroy; override;
  // vrac nhodnou poloku, ale s polem nic nedl
  procedure GetWord(var I : TIndexItem);
  // odstran z pole poloku na pozici indexu; mn Count a Remain, nuluje Index
  procedure ExtractActualWord;
end;

// vrac opan md ne je ActualMode
function OppositeMode(M : TActualMode) : TActualMode;

var
  // globln promnn urujc funknost metod tdy TPair a TWordList
  ActualMode : TActualMode;

implementation

uses SysUtils;

//////////////////////////////////////////////////////////////////////////////
// vrac opan md ne je ActualMode
function OppositeMode(M : TActualMode) : TActualMode;
begin
  Result := M;
  if M = amMix then Exit;
  if M = amOrigin then Result := amTrans
  else Result := amOrigin;
end;

//////////////////////////////////////////////////////////////////////////////
//////   TPair   /////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
{$IFDEF debug}
destructor TPair.Destroy;
begin
  ShowMessage('Prv rum pr: '+#13#10+
              'Originl: '+Origin+#13#10+
              'Peklad: '+Trans);
  inherited;
end;
{$ENDIF}

//////////////////////////////////////////////////////////////////////////////
// vrt Origin nebo Trans
function TPair.GetWordList(Mode : TActualMode) : String;
begin
  if Mode = amOrigin then Result := Origin
  else Result := Trans;
end;

//////////////////////////////////////////////////////////////////////////////
// vrt koment k orginlu nebo k pekladu
function TPair.GetComment(Mode :TActualMode) : String;
begin
  if Mode = amOrigin then Result := OriginComment
  else Result := TransComment;
end;

//////////////////////////////////////////////////////////////////////////////
// vrt poet slov v originle nebo v pekladu
function TPair.GetNum(Mode : TActualMode) : integer;
var
  I : integer;
  PomS : string;
begin
  case Mode of
    amOrigin,
    amMix   : PomS := Origin;
    amTrans :  PomS := Trans;
  end;

  if PomS = '' then Result := 0
  else Result := 1;

  // -1, protoe pokud je na poslednm mst WordSeparator, je tam przdn slovo
  for I := 1 to Length(PomS)-1 do
    if PomS[I] = WordSeparator then
      Result := Result + 1;

  if Mode = amMix then
    Result := Result + GetNum(amTrans);
end;

//////////////////////////////////////////////////////////////////////////////
// vrt slovo z originlu nebo pekladu
function TPair.GetWord(Mode : TActualMode; Index : integer) : String;
var
  I, S, L, P : integer;
  pom, PomS : String;
begin
  Result := '';
  if Index > GetNum(Mode) then
    Exit;

  case Mode of
    amOrigin:  PomS := Origin;
    amTrans :  PomS := Trans;
    amMix : Exit;
  end;

  pom := PomS + WordSeparator;
  S := 1;
  for I := 1 to Index do
  begin
    P := Pos(WordSeparator, pom);
    S := S + P;
    pom := copy(pom, P+1, Length(pom));
  end;
  L := Pos(WordSeparator, pom);
  Result := copy(PomS, S, L-1);
end;

//////////////////////////////////////////////////////////////////////////////
// hled slovo S v originle nebo pekladu
function TPair.Find(Mode : TActualMode; S : String) : boolean;
var
  I : integer;
begin
  Result := false;
  for I := 0 to GetNum(Mode) - 1 do
  begin
    Result := S = GetWord(Mode,I);
    if Result then Break;
  end;
end;

//////////////////////////////////////////////////////////////////////////////
// pid slovo S na konec originlu nebo pekladu
{procedure TPair.Append(Mode : TActualMode; S : String);
var
  PomS : String;
begin
  if Mode = amOrigin then PomS := Origin
  else PomS := Trans;
// !!!!!!!!!!!
  if Pos(WordSeparator + S + WordSeparator,
     WordSeparator + PomS + WordSeparator) > 0 then Exit;
// !!!!!!!!!!!
  try
    if PomS = '' then
       PomS := S
    else
      PomS := PomS + WordSeparator + S;
  finally
    if Mode = amOrigin then Origin := PomS
    else Trans := PomS;
  end;
end;
}
//////////////////////////////////////////////////////////////////////////////
//////   TWordList   /////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
constructor TWordList.Create;
begin
  inherited;
  FError := 0;
end; // Create

//////////////////////////////////////////////////////////////////////////////
// uvoln pam alokovanou polokami
destructor TWordList.Destroy;
var
  I, Max : integer;
  Pair : TPair;
begin
{$IFDEF debug}
  ShowMessage('Prv se ru seznam');
{$ENDIF}
  Max := Count - 1;
  for I := 0 to Max do
  begin
    Pair := Items[0];
    Remove(Pair);
    Pair.Free;
  end;
  inherited;
end;

{
//////////////////////////////////////////////////////////////////////////////
// zkopruje se do jinho seznamu; parametr u mus bt vytvoen
procedure TWordList.CopyList(var L : TWordList);
var
  Pair, pom : TPair;
  I : integer;
begin
  if L = nil then Exit;
  for I := 0 to Count - 1 do
  begin
    pom := Items[I];

    Pair := TPair.Create;
    Pair.Origin := pom.Origin;
    Pair.Trans := pom.Trans;

    Pair.OriginComment := pom.OriginComment;
    Pair.TransComment := pom.TransComment;

    L.Add(Pair);
  end;
end;
}
//////////////////////////////////////////////////////////////////////////////
// statick funkce pro vracen textovho popisu chyby
class function TWordList.GetErrorText(E : integer) : String;
const
  Err : array [0..6] of String =
    (
     'Bez chyby.',                      // 0
     'Neukonen pr slovek.',        // 1
     'Chyb originln st slovka.', // 2
     'Chyb peloen st slovka.',  // 3
     'Syntaktick chyba.',              // 4
     'Chyba pi otevrn souboru',     // 5
     'Tato chyba nikdy neme nastat.'+#13#10+
     'Pokud pesto nastala, vypnte pota a rychle utkejte pry!'
    );
begin
  if E > 6 then E := 6;
  Result := Err[E];
end; // GetErrorText

//////////////////////////////////////////////////////////////////////////////
// vrac aktuln kd chyby
function TWordList.GetError : integer;
begin
  Result := FError;
end; // GetError

//////////////////////////////////////////////////////////////////////////////
// pid pr do seznamu
function TWordList.Add(Item : TPair) : Integer;
begin
  Result := inherited Add(Item);
end; // Add

//////////////////////////////////////////////////////////////////////////////
// odstran pr ze seznamu
function TWordList.Remove(Item: TPair): Integer;
begin
  Result := inherited Remove(Item);
end; // Remove

//////////////////////////////////////////////////////////////////////////////
// ulo seznam do souboru
procedure TWordList.SaveToFile(FileName : String);
var
  I : integer;
  F : TextFile;
  Pair : TPair;
begin
{$I-}
  AssignFile(F,FileName);
  Rewrite(F);
  if IoResult <> 0 then
{$I+}
  begin
    FError := 5;   // Chyba pi otevrn souboru
    Exit;
  end;
  try
    for I := 0 to Count - 1 do
    begin
      Pair := Items[I];          // vbr poloky
      Write(F,Pair.Origin);     // zpis originln sti
      if Pair.OriginComment <> '' then
      begin
        Write(F,CommentPrep);
        Write(F,Pair.OriginComment);
      end;
      Write(F,SingleSeparator);  // oddlova originln a peloen sti
      Write(F,Pair.Trans);      // zpis peloen sti
      if Pair.TransComment <> '' then
      begin
        Write(F,CommentPrep);
        Write(F,Pair.TransComment);
      end;
      Write(F,PairSeparator);    // oddlova zznam
    end;
  finally
    CloseFile(F);
  end;
end; // SaveToFile

//////////////////////////////////////////////////////////////////////////////
// vrac seznam slovek, tj. originl nebo peklad
function TWordList.GetWord(var F : TextFile) : String;
const
  SeparatorSet : set of char = [#19, #10, #12];  // !!!
begin
  Result := '';
  if Eof(F) then Exit;
  repeat
    Read(F, ch);
    if not (ch in SeparatorSet) then
      Result := Result + ch;
  until Eof(F) or (ch in SeparatorSet);
end; // GetWord

//////////////////////////////////////////////////////////////////////////////
// nate seznam ze souboru
procedure TWordList.LoadFromFile(FileName : String);
var
  F : TextFile;
  Pair : TPair;
{################################################}
procedure GetPair(var Pair : TPair);
begin
  Pair.Origin := GetWord(F);
  if Pair.Origin = '' then
  begin
    FError := 2; Abort;   // chyb originl
  end;
  if Eof(F) then
  begin
    FError := 1; Abort;   // slovko kon v plce
  end;
  if ch = PairSeparator then
  begin
    FError := 3; Abort;   // chyb peklad
  end;
  if ch = CommentPrep then
    Pair.OriginComment := GetWord(F);

  Pair.Trans := GetWord(F);
  if Pair.Trans = '' then
  begin
    FError := 3; Abort;
  end;
  if ch = SingleSeparator then
  begin
    FError := 4; Abort;  // syntaktick chyba
  end;
  if ch = CommentPrep then
    Pair.TransComment := GetWord(F);
end; // GetPair

begin // LoadFromFile
{$I-}
  AssignFile(F, FileName);
  Reset(F);
  if IoResult <> 0 then
{$I+}
  begin
    FError := 5;   // Chyba pi otevrn souboru
    Exit;
  end;
  try
    while not Eof(F) do
    begin
      Pair := TPair.Create;
      GetPair(Pair);
      Add(Pair);
    end;
  finally
    CloseFile(F);
  end;
end; // LoadFromFile

//////////////////////////////////////////////////////////////////////////////
// vrt poet originl nebo peklad
function TWordList.GetCount(Mode : TActualMode) : integer;
var
  I : integer;
begin
  Result := 0;
  for I := 0 to Count-1 do
    Result := Result + TPair(Items[I]).GetNum(Mode);
end;

//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
constructor TIndexList.Create(const L : TWordList; Test : TActualMode;
  Max : integer);
var
  WordCounter, CoupleIndex, WordNumber : integer;
  PP : TPair;
begin
  Maximum := Max;
  Count := L.GetCount(Test); // vrt celkov poet slov
  Size := Count * SizeOf(TIndexItem);
  GetMem(IndexArray, Size);
  WordCounter := 0;
  for CoupleIndex := 0 to L.Count - 1 do
  begin
    PP := L.Items[CoupleIndex];
    if Test = amMix then
    begin
      for WordNumber := 0 to PP.GetNum(amOrigin) - 1 do
      begin
        with IndexArray^[WordCounter] do
        begin
          Number := WordNumber;
          Mode := amOrigin;
          WIndex := CoupleIndex;
        end;
        inc(WordCounter);
      end;
      for WordNumber := 0 to PP.GetNum(amTrans) - 1 do
      begin
        with IndexArray^[WordCounter] do
        begin
          Number := WordNumber;
          Mode := amTrans;
          WIndex := CoupleIndex;
        end;
        inc(WordCounter);
      end;
    end else
    begin
      for WordNumber := 0 to PP.GetNum(Test) - 1 do
      begin
        with IndexArray^[WordCounter] do
        begin
          Number := WordNumber;
          Mode := Test;
          WIndex := CoupleIndex;
        end;
        inc(WordCounter);
      end;
    end;
  end;
  Randomize;
  if (Maximum > 0) and (Maximum < Count) then
    while Maximum < Count do
    begin
      Index := Random(Count);
      ExtractActualWord;
    end;

end;

destructor TIndexList.Destroy;
begin
  FreeMem(IndexArray, Size);
  inherited;
end;

procedure TIndexList.GetWord(var I : TIndexItem);
begin
  Index := Random(Count);
  I := IndexArray^[Index];
end;

procedure TIndexList.ExtractActualWord;
begin
  if Count <= 0 then Exit;
  IndexArray^[Index] := IndexArray^[Count - 1];
  Count := Count - 1;
  Index := 0;
end;

end.

