/////////////////////////////////////////////////////////////////////////////
//  Projekt: STest - testovn slovek a tvorba test                     //
//  Verze: 2                                                               //
//  Autor: David Martinek                                                  //
//  Adresa: Majakovskho 2148/60, Karvin 7, 734 01                        //
//                                                                         //
//  Funkce modulu: Dialog pro zznam nastaven programu do Registran     //
//    databze                                                             //
//  Datum: 8.9.1999                                                       //
/////////////////////////////////////////////////////////////////////////////
unit ZaznFrm;

interface

uses Windows, SysUtils, Forms, Controls, StdCtrls, Buttons, ExtCtrls, Registry,
  Dialogs, Classes, Spin,
  WordList;

type
  TLogOpt = (loFileName, loMark, loSuccess, loErrorCount, loWordCount, loMode);
  TLogOptSet = set of TLogOpt; // mnoina urujc co se m zaznamenat

  // pracovn zznam pro uloen do Registru
  TLogOptions = record
    FileName : String;
    MaxLen : integer;
    OptSet : TLogOptSet;
    NormalMode : TActualMode;
  end;

  TZaznForm = class(TForm)
    OKBtn: TButton;
    CBtn: TButton;
    OpenDialog1: TOpenDialog;
    SE: TSpinEdit;
    MaxCB: TCheckBox;
    GroupBox1: TGroupBox;
    FileNameCB: TCheckBox;
    ModeCB: TCheckBox;
    ErrorCountCB: TCheckBox;
    WordCountCB: TCheckBox;
    SuccessCB: TCheckBox;
    MarkCB: TCheckBox;
    L1: TLabel;
    Edit1: TEdit;
    SpeedButton1: TSpeedButton;
    procedure FormShow(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure Edit1KeyPress(Sender: TObject; var Key: Char);
    procedure MaxCBClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

// nate z Registry nastaven
procedure ReadLogOptions;

// zape nastaven do Registry
procedure WriteLogOptions;

// pid do souboru zznam o zkouce
procedure WriteLogRecord(const FName, Mark : string; Success : single; ErrorCount, WordCount : integer);

// zobraz dialogov okno
procedure ShowLogForm;

var
  LogOptions : TLogOptions;

implementation

{$R *.DFM}

const  // textov konstanty - zapisuj se do Registru
  SRegPath = 'Software\STest';
  SLogOptions = 'LogOptions';
  SloFileName = 'loFileName';
  SloMark = 'loMark';
  SloSuccess = 'loSuccess';
  SloErrorCount = 'loErrorCount';
  SloWordCount = 'loWordCount';
  SloMode = 'loMode';
  SLogFileName = 'LogFileName';
  SNormalMode = 'NormalMode';
  SMaxLen = 'MaxLen';

//////////////////////////////////////////////////////////////////////////////
// nate z Registry nastaven
procedure ReadLogOptions;
var
  Reg : TRegIniFile;
begin
  Reg := TRegIniFile.Create(SRegPath);
  try
    if Reg.ReadBool(SLogOptions,SloFileName, loFileName in LogOptions.OptSet) then LogOptions.OptSet := LogOptions.OptSet + [loFileName]
    else LogOptions.OptSet := LogOptions.OptSet - [loFileName];
    if Reg.ReadBool(SLogOptions,SloMark, loMark in LogOptions.OptSet) then LogOptions.OptSet := LogOptions.OptSet + [loMark]
    else LogOptions.OptSet := LogOptions.OptSet - [loMark];
    if Reg.ReadBool(SLogOptions,SloSuccess, loSuccess in LogOptions.OptSet) then LogOptions.OptSet := LogOptions.OptSet + [loSuccess]
    else LogOptions.OptSet := LogOptions.OptSet - [loSuccess];
    if Reg.ReadBool(SLogOptions,SloErrorCount, loErrorCount in LogOptions.OptSet) then LogOptions.OptSet := LogOptions.OptSet + [loErrorCount]
    else LogOptions.OptSet := LogOptions.OptSet - [loErrorCount];
    if Reg.ReadBool(SLogOptions,SloWordCount, loWordCount in LogOptions.OptSet) then LogOptions.OptSet := LogOptions.OptSet + [loWordCount]
    else LogOptions.OptSet := LogOptions.OptSet - [loWordCount];
    if Reg.ReadBool(SLogOptions,SloMode, loMode in LogOptions.OptSet) then LogOptions.OptSet := LogOptions.OptSet + [loMode]
    else LogOptions.OptSet := LogOptions.OptSet - [loMode];
    LogOptions.FileName := Reg.ReadString(SLogOptions,SLogFileName,LogOptions.FileName);
    LogOptions.NormalMode := TActualMode(Reg.ReadInteger(SLogOptions,SNormalMode,ord(LogOptions.NormalMode)));
    LogOptions.MaxLen := Reg.ReadInteger(SLogOptions,SMaxLen,LogOptions.MaxLen);
  finally
    Reg.Destroy;
  end;
end;

//////////////////////////////////////////////////////////////////////////////
// zape nastaven do Registry
procedure WriteLogOptions;
var
  Reg : TRegIniFile;
begin
  Reg := TRegIniFile.Create(SRegPath);
  try
    Reg.WriteBool(SLogOptions,SloFileName, loFileName in LogOptions.OptSet);
    Reg.WriteBool(SLogOptions,SloMark, loMark in LogOptions.OptSet);
    Reg.WriteBool(SLogOptions,SloSuccess, loSuccess in LogOptions.OptSet);
    Reg.WriteBool(SLogOptions,SloErrorCount, loErrorCount in LogOptions.OptSet);
    Reg.WriteBool(SLogOptions,SloWordCount, loWordCount in LogOptions.OptSet);
    Reg.WriteBool(SLogOptions,SloMode, loMode in LogOptions.OptSet);
    Reg.WriteString(SLogOptions,SLogFileName,LogOptions.FileName);
    Reg.WriteInteger(SLogOptions,SNormalMode,ord(LogOptions.NormalMode));
    Reg.WriteInteger(SLogOptions,SMaxLen,LogOptions.MaxLen);
  finally
    Reg.Destroy;
  end;
end;

//////////////////////////////////////////////////////////////////////////////
// pid do souboru zznam o zkouce
procedure WriteLogRecord(const FName, Mark : string; Success : single; ErrorCount, WordCount : integer);
var
  F : TextFile;
  S : string;
begin
  AssignFile(F, LogOptions.FileName);  // piad soubor
{$I-}
  Append(F);   // oteven souboru pro pidvn na konec
  if IoResult <> 0 then    // chyba pi otvrn souboru
  begin
    if Application.MessageBox(PChar('Soubor pro zznam vsledku ' +
      LogOptions.FileName +
      ' pravdpodobn neexistuje.' +#13#10+'Mm jej vytvoit?'),
      'Problm s otevenm souboru',
      MB_ICONWARNING + MB_YESNO + MB_DEFBUTTON1) = IDYES
    then Rewrite(F)  // vytvoen souboru a oteven pro zpis
    else Exit;
  end;
{$I+}
  try
    Writeln(F, DateTimeToStr(Now));   // prvn dek je datum

    if (loFileName in LogOptions.OptSet) or
       (loMode in LogOptions.OptSet) then
      Write(F, 'TEST: ');

    if loFileName in LogOptions.OptSet then
      Write(F, ExtractFileName(FName));

    if (loFileName in LogOptions.OptSet)and
       (loMode in LogOptions.OptSet) then
      Write(F, ', ');

    if loMode in LogOptions.OptSet then
      case LogOptions.NormalMode of
        amOrigin : Writeln(F, 'originl');
        amTrans  : Writeln(F, 'peklad');
        amMix    : Writeln(F, 'vechno');
      end
    else if loFileName in LogOptions.OptSet then
      Writeln(F);

    if loWordCount in LogOptions.OptSet then
      Writeln(F, 'SLOV: ' + IntToStr(WordCount));

    if loErrorCount in LogOptions.OptSet then
      Writeln(F, 'CHYB: ' + IntToStr(ErrorCount));

    if loSuccess in LogOptions.OptSet then
    begin
      Str(Success:3:2, S);
      Writeln(F, 'SPCH: ' + S + ' %');
    end;

    if loMark in LogOptions.OptSet then
      Writeln(F, 'ZNMKA: ' + Mark);

    Writeln(F);
  finally
    CloseFile(F);
  end;
end;

//////////////////////////////////////////////////////////////////////////////
// zobraz dialogov okno
procedure ShowLogForm;
var
  ZaznForm: TZaznForm;
begin
  ZaznForm := TZaznForm.Create(Application);
  try
    ZaznForm.ShowModal;
  finally
    ZaznForm.Release;
  end;
end;

//////////////////////////////////////////////////////////////////////////////
procedure TZaznForm.FormShow(Sender: TObject);
begin
  FileNameCB.Checked := loFileName in LogOptions.OptSet;
  ModeCB.Checked := loMode in LogOptions.OptSet;
  MarkCB.Checked := loMark in LogOptions.OptSet;
  SuccessCB.Checked := loSuccess in LogOptions.OptSet;
  ErrorCountCB.Checked := loErrorCount in LogOptions.OptSet;
  WordCountCB.Checked := loWordCount in LogOptions.OptSet;
  Edit1.Text := LogOptions.FileName;
  SE.Value := LogOptions.MaxLen;
  if LogOptions.MaxLen <= 0 then MaxCB.Checked := false;
  SE.Enabled := MaxCB.Checked;
end;

//////////////////////////////////////////////////////////////////////////////
procedure TZaznForm.OKBtnClick(Sender: TObject);
begin
  if FileNameCB.Checked then LogOptions.OptSet := LogOptions.OptSet + [loFileName]
  else LogOptions.OptSet := LogOptions.OptSet - [loFileName];
  if ModeCB.Checked then LogOptions.OptSet := LogOptions.OptSet + [loMode]
  else LogOptions.OptSet := LogOptions.OptSet - [loMode];
  if MarkCB.Checked then LogOptions.OptSet := LogOptions.OptSet + [loMark]
  else LogOptions.OptSet := LogOptions.OptSet - [loMark];
  if SuccessCB.Checked then LogOptions.OptSet := LogOptions.OptSet + [loSuccess]
  else LogOptions.OptSet := LogOptions.OptSet - [loSuccess];
  if ErrorCountCB.Checked then LogOptions.OptSet := LogOptions.OptSet + [loErrorCount]
  else LogOptions.OptSet := LogOptions.OptSet - [loErrorCount];
  if WordCountCB.Checked then LogOptions.OptSet := LogOptions.OptSet + [loWordCount]
  else LogOptions.OptSet := LogOptions.OptSet - [loWordCount];
  if MaxCb.Checked then LogOptions.MaxLen := SE.Value
  else LogOptions.MaxLen := 0;
end;

//////////////////////////////////////////////////////////////////////////////
procedure TZaznForm.SpeedButton1Click(Sender: TObject);
begin
  OpenDialog1.FileName := LogOptions.FileName;
  if OpenDialog1.Execute then
  begin
    Edit1.Text := OpenDialog1.FileName;
    LogOptions.FileName := OpenDialog1.FileName;
  end;
end;

//////////////////////////////////////////////////////////////////////////////
procedure TZaznForm.Edit1KeyPress(Sender: TObject; var Key: Char);
begin
  if (Key <> #37) and (Key <> #39) then Key := #0;
end;

procedure TZaznForm.MaxCBClick(Sender: TObject);
begin
  SE.Enabled := MaxCB.Checked;
end;

end.
